SystemOrganization addCategory: #'Polymorph-Geometry'!

!Point methodsFor: '*Polymorph-Geometry' stamp: 'gvc 10/31/2006 11:01'!
directionToLineFrom: p1 to: p2
	"Answer the direction of the line from the receiver
	position.
	< 0 => left (receiver to right)
	= => on line
	> 0 => right (receiver to left)."

	^((p2 x - p1 x) * (self y - p1 y)) -
		((self x - p1 x) * (p2 y - p1 y))! !

Object subclass: #Shape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Geometry'!

Shape subclass: #PathShape
	instanceVariableNames: 'bounds vertices'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Geometry'!

!PathShape class methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 11:08'!
vertices: aCollection
	"Answer a new instance of the receiver with the
	given vertices."

	^self new
		vertices: aCollection! !

!PathShape methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 10:40'!
addVertex: aPoint
	"Add a vertex to the path."

	self vertices add: aPoint.
	self basicBounds ifNotNil: [
		self bounds: (self bounds quickMergePoint: aPoint)] ! !

!PathShape methodsFor: 'accessing' stamp: 'gvc 10/31/2006 10:36'!
basicBounds
	"Answer the bounds of the receiver without lazy calculation."

	^bounds! !

!PathShape methodsFor: 'accessing' stamp: 'gvc 10/31/2006 10:26'!
bounds
	"Answer the bounds of the receiver."

	^bounds ifNil: [bounds _ self calculatedBounds]! !

!PathShape methodsFor: 'accessing' stamp: 'gvc 10/31/2006 10:25'!
bounds: anObject
	"Set the value of bounds"

	bounds := anObject! !

!PathShape methodsFor: 'as yet unclassified' stamp: 'gvc 6/25/2007 13:52'!
calculatedBounds
	"Answer the bounds of the receiver calculated from the
	receiver's vertices."

	|tl br|
	self vertices ifEmpty: [^nil].
	tl := br := self vertices first.
	self vertices allButFirstDo: [:v |
		tl := tl min: v.
		br := br max: v].
	^tl corner: br + 1! !

!PathShape methodsFor: 'as yet unclassified' stamp: 'gvc 6/25/2007 14:03'!
containsPoint: aPoint
	"Answer whether the receiver contains the given point."

	(self basicContainsPoint: aPoint) ifFalse: [^false].
	self segmentsDo: [:p1 :p2 |
		(aPoint onLineFrom: p1 to: p2 within: 0) ifTrue: [^true]].
	^false! !

!PathShape methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 10:35'!
initialize
	"Initialize the receiver."

	super initialize.
	self
		vertices: OrderedCollection new! !

!PathShape methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 10:47'!
segmentsDo: aBlock
	"Evaluate the two-argument block with each vertex and its successor."

	self vertices size < 2 ifTrue: [^self].
	1 to: self vertices size - 1 do: [:i |
		aBlock
			value: (self vertices at: i)
			value: (self vertices at: i + 1)]! !

!PathShape methodsFor: 'accessing' stamp: 'gvc 10/31/2006 10:24'!
vertices
	"Answer the value of vertices"

	^ vertices! !

!PathShape methodsFor: 'accessing' stamp: 'gvc 10/31/2006 11:08'!
vertices: aCollection
	"Set the value of vertices."

	vertices := aCollection asOrderedCollection.
	self bounds: nil! !

PathShape subclass: #Polygon
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Geometry'!

!Polygon methodsFor: 'as yet unclassified' stamp: 'gvc 6/25/2007 14:43'!
containsPoint: aPoint
	"Answer whether the receiver contains the given point."

	|wind|
	(self basicContainsPoint: aPoint) ifFalse: [^false].
	wind := 0.
	self segmentsDo: [:p1 :p2 |
		p1 y <= aPoint y
			ifTrue: [p2 y = aPoint y
						ifTrue: [(aPoint directionToLineFrom: p1 to: p2) = 0
							ifTrue: [^true]]
						ifFalse: [(p2 y > aPoint y and: [(aPoint directionToLineFrom: p1 to: p2) > 0])
							ifTrue: [wind := wind + 1]]]
			ifFalse: [p2 y = aPoint y
						ifTrue: [(aPoint directionToLineFrom: p1 to: p2) = 0
								ifTrue: [^true]].
					(p2 y < aPoint y and: [(aPoint directionToLineFrom: p1 to: p2) < 0])
							ifTrue: [wind := wind - 1]]].
	^wind ~= 0! !

!Polygon methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 10:52'!
segmentsDo: aBlock
	"Evaluate the two-argument block with each vertex and its successor."

	self vertices size < 2 ifTrue: [^self].
	super segmentsDo: aBlock.
	aBlock
		value: self vertices last
		value: self vertices first! !

!Shape methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 10:48'!
basicContainsPoint: aPoint
	"Answer whether the receiver contains the given point."

	^self bounds containsPoint: aPoint! !

!Shape methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 10:25'!
bounds
	"Answer a rectangle that encloses the receiver."

	self subclassResponsibility! !

!Shape methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 10:48'!
containsPoint: aPoint
	"Answer whether the receiver contains the given point."

	^self basicContainsPoint: aPoint! !

!Rectangle methodsFor: '*Polymorph-Geometry' stamp: 'gvc 6/25/2007 14:00'!
quickMergePoint: aPoint
	"Answer the receiver if it encloses the given point or the expansion of the
	receiver to do so if it doesn't. "

	| useRcvr minX maxX minY maxY |
	useRcvr := true.
	minX := aPoint x < origin x ifTrue: [useRcvr := false. aPoint x] ifFalse: [origin x].
	maxX := aPoint x >= corner x ifTrue: [useRcvr := false. aPoint x + 1] ifFalse: [corner x].
	minY := aPoint y < origin y ifTrue: [useRcvr := false. aPoint y] ifFalse: [origin y].
	maxY := aPoint y >= corner y ifTrue: [useRcvr := false. aPoint y + 1] ifFalse: [corner y].
	^useRcvr
		ifTrue: [self]
		ifFalse: [minX@minY corner: maxX@maxY]
! !
